﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using System.Globalization;
using System.IO;
using PasswordCombinationGenerator.Properties;
using UCPasswordCombinationGenerator;

namespace PasswordCombinationGenerator
{
    public partial class PasswordCombinationGenerator : Form
    {
        public PasswordCombinationGenerator()
        {
            InitializeComponent();
        }

        private void PasswordCombinationGenerator_Load(object sender, EventArgs e)
        {
            
        }

        private void btnSettings_Click(object sender, EventArgs e)
        {
            ShowSettings();
        }

        private void ShowSettings()
        {
            Settings s = new Settings();
            s.ShowDialog();
        }

        private void btnCopyCurrentResults_Click(object sender, EventArgs e)
        {
            try
            {
                Clipboard.SetText(txtCurrentResults.Text, TextDataFormat.UnicodeText);
                tslStatus.Text = "Copied Current Results to clipboard.";
            }
            catch
            {
            }
        }

        private void btnCopyAllResults_Click(object sender, EventArgs e)
        {
            try
            {
                Clipboard.SetText(txtAllResults.Text, TextDataFormat.UnicodeText);
                tslStatus.Text = "Copied All Results to clipboard.";
            }
            catch
            {
            }
        }

        string[] separatorLine = { "\r\n" };

        private void RunResults()
        {
            if (tabControl1.SelectedIndex == 0)
            {
                RunResultsWorkflow1();
            }
        }

        private void RunResultsWorkflow1()
        {
            
            tokenInputPanel1.RemoveBlankLinesFromAll();
            // we need our sets as object[]s - create an object array 

            // count the input panels
            int ipCount = 0;
            foreach (Control p in tokenInputPanel1.Controls)
            {
                if (p.GetType() == typeof(Panel))
                {
                    // we have the panel, and can now count the inputs.
                    ipCount = p.Controls.Count;
                    break;
                }
            }
            object[][] o = new object[ipCount][];

            // Fill the object array
            for (int i = 0; i < ipCount; i++)
            {
                o[i] = tokenInputPanel1.GetInputLines(i); 
            }

            // pass the object array to our cartesian product method
            var cartesianProduct = CartesianProduct(o);
            StringBuilder sb = new StringBuilder();
            foreach (var item in cartesianProduct)
            {
                try
                {
                    Console.WriteLine(string.Join("\t", item));
                    sb.AppendLine(string.Format(txtSymbolString.Text, item));
                }
                catch
                {
                    MessageBox.Show("The number of inputs and your Symbol String do not match. Please check it and try again.", "Symbol String Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                }
            }
            txtCurrentResults.Text = sb.ToString();
            lblCurrentResults.Text = string.Format("Current Results ({0})", (txtCurrentResults.Lines.Length-1).ToString());
        }


        // Cartesian product stuff
        #region Cartesian product stuff

        public static IEnumerable<object[]> CartesianProduct(params object[][] inputs)
        {
            return inputs.Aggregate(
                (IEnumerable<object[]>)new object[][] { new object[0] },
                (soFar, input) =>
                    from prevProductItem in soFar
                    from item in input
                    select prevProductItem.Concat(new object[] { item }).ToArray());
        }

        // From http://www.interact-sw.co.uk/iangblog/2010/08/02/linq-cartesian-4
        public static IEnumerable<IEnumerable<T>> CartesianProductT<T>(
    IEnumerable<IEnumerable<T>> inputs)
        {
            return inputs.Aggregate(
                EnumerableFrom(Enumerable.Empty<T>()),
                (soFar, input) =>
                    from prevProductItem in soFar
                    from item in input
                    select prevProductItem.Append(item));
        }




        private static IEnumerable<T> EnumerableFrom<T>(T item)
        {
            return new T[] { item };
        }
        #endregion


        private string[] Processed(int size)
        {
            string[] result = new string[size];

            // Fill up the array
            for (int i = 0; i < size; i++)
            {
                result[i] = txtSymbolString.Text;
            }

            return result;
        }


        private int TotalPossibilities(int[] linesPerInput)
        {
            int result = 1;
            foreach (int i in linesPerInput)
            {
                if (i != 0) // We ingore empty inputs
                {
                    result *= i;
                }
            }
            return result;
        }

        private int[] LinesPerInput(int inputCount)
        {
            int[] result = new int[inputCount];

            foreach (Control c in tokenInputPanel1.Controls)
            {
                // This is the complete control. We are only interested in the TokenInput controls
                if (c.GetType() == (new Panel()).GetType())
                {
                    // We now have the panel. We need to get the TokenInputs
                    int index = 0;
                    foreach (Control d in c.Controls)
                    {
                        // d here is a TokenInput. We only need to count them
                        //result[index] = ((TokenInput)d).txtValue.Lines.Count();
                        index++;
                    }
                }
            }

            return result;
        }

        private int InputCount()
        {
            int result = 0;
            foreach (Control c in tokenInputPanel1.Controls)
            {
                // This is the complete control. We are only interested in the TokenInput controls
                if (c.GetType() == (new Panel()).GetType())
                {
                    // We now have the panel. We need to get the TokenInputs
                    foreach (Control d in c.Controls)
                    {
                        // d here is a TokenInput. We only need to count them
                        result++;
                    }
                }
            }

            return result;
        }


        private void btnZoomCurrent_Click(object sender, EventArgs e)
        {
            ZoomTextbox(txtCurrentResults);
        }

        private void ZoomTextbox(TextBox box)
        {
            using (var form = new ZoomView(box.Text))
            {
                var result = form.ShowDialog();
                if (result == DialogResult.OK)
                {
                    box.Text = form.returnValue;
                }
            }
        }

        private void btnZoomAll_Click(object sender, EventArgs e)
        {
            ZoomTextbox(txtAllResults);
        }

        private void tbRunResults_Click(object sender, EventArgs e)
        {
            RunResults();
        }


        #region File appendage
        private string CreateFileAppendage(string appendage)
        {
            string result = appendage;
            //switch (Properties.Settings.Default.saveAppend)
            //{
            //    case 0:
            //        appendage = "." + LongFileDateAppendage();
            //        break;

            //    case 1:
            //        appendage = "." + DateTime.Now.Ticks.ToString();
            //        break;

            //    case 2:
            //        appendage = string.Empty;
            //        break;

            //    default:
            //        appendage = "." + LongFileDateAppendage();
            //        break;
            //}
            return appendage;
        }

        private string LongFileDateAppendage()
        {
            return DateTime.Now.Year.ToString() + "-" + DateTime.Now.Month.ToString("00") + "-" + DateTime.Now.Day.ToString("00") + "-" + DateTime.Now.Hour.ToString("00") + "-" + DateTime.Now.Minute.ToString("00") + "-" + DateTime.Now.Second.ToString("00") + "-" + DateTime.Now.Millisecond.ToString("000");
        }
        #endregion

        private bool SaveFile(string text, string filename)
        {
            bool result = false;
            try
            {
                TextWriter tw = new StreamWriter(filename);
                tw.Write(text);
                tw.Flush();
                tw.Close();
                tw.Dispose();
                result = true;
            }
            catch
            {
                result = false;
            }
            return result;
        }

        private void btnCopyCurrentToAll_Click(object sender, EventArgs e)
        {
            if (txtAllResults.Text == string.Empty)
            {
                txtAllResults.Text = txtCurrentResults.Text.Trim();
            }
            else
            {
                txtAllResults.Text = txtAllResults.Text.Trim() + "\r\n" + txtCurrentResults.Text.Trim();
            }
            tslStatus.Text = "Appended Current Results to All Results";
        }

        private void btnRunResults_Click(object sender, EventArgs e)
        {
            RunResults();
        }


        private void PasswordCombinationGenerator_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void txtSymbolString_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void tsbAbout_Click(object sender, EventArgs e)
        {
            About about = new About();
            about.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }


    static class AppendExtension
    {
        public static IEnumerable<T> Append<T>(this IEnumerable<T> that, T item)
        {
            IEnumerable<T> itemAsSequence = new T[] { item };
            return that.Concat(itemAsSequence);
        }
    }

}
