﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace UCPasswordCombinationGenerator
{
    public partial class TokenInput : UserControl
    {
        public TokenInput()
        {
            InitializeComponent();
            txtValue.Text = string.Empty;
        }

        public TokenInput(int count)
        {
            InitializeComponent();
            txtValue.Text = string.Empty;
            this.lblNumber.Text = count.ToString();
            this.Count = count;
        }

        public TokenInput(int count, int height)
        {
            InitializeComponent();
            txtValue.Text = string.Empty;
            this.lblNumber.Text = count.ToString();
            this.Count = count;
            int diff = this.Height - txtValue.Height;
            this.Height = height;
            txtValue.Height = height - diff;

        }

        private void TokenInput_Load(object sender, EventArgs e)
        {
            
        }

        public int ReduceHeight(int amount)
        {
            int result = 0;

            if (this.Height >= 125 && amount >0)
            {
                this.Height -= amount;
                txtValue.Height -= amount;
                result = amount;
            }
            else if (amount < 0)
            {
                this.Height -= amount;
                txtValue.Height -= amount;
                result = amount;
            }


            return result;
        }

        private int _count = 0;

        public int Count
        {
            get { return _count; }
            set 
            { 
                _count = value;
                lblNumber.Text = value.ToString();
                Application.DoEvents();
            }
        }

        private string _value = string.Empty;
        public string Value
        {
            get { return _value; }
            set { _value = value; }
        }

        private int _lineCount = 0;

        public int LineCount
        {
            get { return _lineCount; }
            set { _lineCount = value; }
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            Value = txtValue.Text;
            if (!string.IsNullOrWhiteSpace(txtValue.Text))
            {
                Clipboard.SetText(txtValue.Text, TextDataFormat.UnicodeText);
            }
        }

        private void btnDuplicateLastLine_Click(object sender, EventArgs e)
        {
            string[] lines = txtValue.Lines;
            int c = lines.Length - 1;
            while (true)
            {
                if (c >= 0 && lines[c] != string.Empty)
                {
                    if (lines[lines.Length - 1].Trim() == string.Empty)
                    {
                        txtValue.Text += lines[c];
                    }
                    else
                    {
                        txtValue.Text += "\r\n" + lines[c];
                    }
                    break;
                }
                else if (c < 0)
                {
                    break;
                }
                c--;
            }
            if (txtValue.Text.Length > 0)
            {
                txtValue.SelectionStart = txtValue.Text.Length - 1;
            }
            else
            {
                txtValue.SelectionStart = 0;
            }
            txtValue.ScrollToCaret();
        }

        private void btnRemoveBlankLines_Click(object sender, EventArgs e)
        {
            RemoveBlankLines();
        }

        public void RemoveBlankLines()
        {
            txtValue.Text = RemoveEmptyLines(txtValue.Text).Trim();
        }

        public static string RemoveEmptyLines(string hasEmptyLines)
        {
            string emptyLinesRemoved = string.Empty;

            string[] lines = hasEmptyLines.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);

            foreach (string line in lines)
            {
                emptyLinesRemoved += line + Environment.NewLine;
            }

            return emptyLinesRemoved;
        }

        private void txtValue_TextChanged(object sender, EventArgs e)
        {
            Value = txtValue.Text;
        }

        private void btnZoomTokenInput_Click(object sender, EventArgs e)
        {
            ZoomTextbox(txtValue);
        }
        private void ZoomTextbox(TextBox box)
        {
            using (var form = new ZoomView(box.Text))
            {
                var result = form.ShowDialog();
                if (result == DialogResult.OK)
                {
                    box.Text = form.returnValue;
                }
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Are you sure you want to delete this? This action is NOT undoable.", "Delete input?", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            if (dr == DialogResult.Yes)
            {
                if (RemoveControlEvent != null)
                {
                    RemoveControlEvent(this, e);
                }
            }
        }

        public delegate void RemoveControlHandler(object sender, EventArgs e);
        public event RemoveControlHandler RemoveControlEvent;

    }
}
